; NES Init Code
; Copyright 2015 Johnathan Roatch
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;

.include "nes.inc"

.export init_reset_handler, init_use_simple_ramvectors

.importzp temp
.importzp nmi_count, tv_type
.importzp ramvectors, ramvectors_nmi, ramvectors_irq, stack_tos

.import rng_init, ppu_clear_oam_nocheck, ppu_set_addr, ppu_forced_blank
.import coredump, main

.import __SYSTEMZP_LOAD__

.segment "CODE"

.proc init_reset_handler
  sei                   ; Disable IRQ
  ldy #%00000000
  sty PPU_CTRL          ; Disable NMI
    ; Stop IRQ and NMI interrupts ASAP.

; COREDUMP NOTE: Delay setting the stack pointer until
;   after the first controller read.

; Putting other sources of interrupts into a known state.
  sty PPU_MASK          ; Disable rendering
  sty APU_DMC_FREQ      ; Disable DMC IRQ
  bit APU_STATUS        ; Acknowledge DMC IRQ
  lda #%01000000
  sta APU_FRAMECNT      ; Disable APU Frame IRQ
  lda #%00001111
  sta APU_SND_CHN       ; Disable DMC playback, initialize other channels
  cld                   ; Turn off decimal mode for compatibility.

; Setting up for the cycle timed loops.
  ;,; ldy #0
  ldx #$100-1
  lda #24
; Acknowledge stray vblank NMI across reset
  bit PPU_STATUS
; It takes one full frame for the PPU to become stable.
; Best practice is to spin twice until the vblank flag of
; PPU_STATUS is set.
  vwait1:
    bit PPU_STATUS
  bpl vwait1

; We have cycles to burn until the second frame's vblank.
; Use this time to detect what tv system we're on.
; PPU_STATUS is checked after these amount of cycles
; but before each of their vblank ends.
;   NTSC NES: 29780 cycles per frame (2273 for vblank)
;   PAL NES:  33247 cycles per frame (7459 for vblank)
;   Dendy:    35464 cycles per frame (2273 for vblank)
; I end up hitting around cycles 30870, 33440, 36020.
  vwait2:
    @multiple_loop:
      @1279_cycles:
        dey
      bne @1279_cycles
      sec
      sbc #1
    bne @multiple_loop
    inx
    lda #2
    bit PPU_STATUS
  bpl vwait2
    ; After the second vblank, we know the PPU has fully stabilized,
    ; and that the TV type is in X.

; Read controller into Y for system boot functions 
; like displaying the unmodified contents of RAM.
system_boot_readpad:
  ;,; lda #$00
  iny ;,; ldy #$01
  sty JOY_STROBE
  sta JOY_STROBE
  @__loop:
    lda JOY_PORT_1
    ora JOY_PORT_2
    and #%00000011
    cmp #1
    tya
    rol
    tay
  bcc @__loop

  ;,; tya
  and #BUTTON_A|BUTTON_B|BUTTON_START|BUTTON_SELECT
  eor #BUTTON_A|BUTTON_B
  beq decode_pad_for_memory_clearing

  stx tv_type
  ldx #$ff
  txs
    ; Finally! We can now touch the RAM and stack pointer.

  jsr init_use_simple_ramvectors
    ; because we have our vectors pointing to ram
    ; we must initalize it before enabling NMI and IRQ.
  cli

  jsr ppu_forced_blank

  ;,; ldy #$00
  jsr ppu_clear_oam_nocheck

  jsr rng_init
    ; I know some bytes in zeropage already got set to a known
    ; state thereby reducing entropy, but hopfully leftover
    ; OAM will have enough anyway.

  lda #0
  ldx #<__SYSTEMZP_LOAD__
  clear_zp_loop:
    dex
    sta 0, x
  bne clear_zp_loop

  ldx #<__SYSTEMZP_LOAD__-2
  stx stack_tos
    ; initalize data stack pointer.

  jsr main
jmp coredump      ; in case of a incorrect rts error from an app.

decode_pad_for_memory_clearing:
  tya
  ;,; ora #BUTTON_A|BUTTON_B
  ldy #4+2
  @__loop:
    dey
    lsr
  bcc @__loop
  lda clear_memory_options-2, y
  iny  ;,; cpy #ff
  beq skip_memory_clear
    ldx #0
    @memory_clear_loop:
      cpy #6
      bne @skip_addr_pat
        txa
      @skip_addr_pat:
      sta $00,x
      sta $0100,x
      sta $0200,x
      sta $0300,x
      sta $0400,x
      sta $0500,x
      sta $0600,x
      sta $0700,x
      inx
    bne @memory_clear_loop
  skip_memory_clear:
jmp coredump

clear_memory_options:
  .byte $ff,$00,$a5
.endproc


;;
; TODO: move to menu.s
;.proc init_start_activity
;  asl
;  tay
;  lda activity_start_vectors+1, y
;  pha
;  lda activity_start_vectors, y
;  pha
;jmp init_use_simple_ramvectors
;
;activity_start_vectors:
;  .addr main-1
;;  .addr button_logger_activity-1
;;  .addr the42melody-1
;.endproc


.proc init_use_simple_ramvectors
  lda #$40                   ; rti opcode
  sta ramvectors_nmi+0       ; lock out NMI for a bit
  sta ramvectors_irq+0       ; null IRQ
; loading last byte first, NMI will become:
;   inc nmi_count
;   rti
  ;,; lda #$40
  sta ramvectors_nmi+2
  lda #nmi_count
  sta ramvectors_nmi+1
  lda #$e6  ; inc zp opcode
  sta ramvectors_nmi+0
rts
.endproc

